#include <Arduino.h>
#include <avr/wdt.h>

#include <RedFly.h>

#include "matrix.hpp"
#include "config.hpp"
#include "redFlyClient.hpp"
#include "serialClient.hpp"

char mode = 0;
unsigned long startS = 0, startW = 0;

void setup() {
  matrix_init();
  pinMode(3, OUTPUT);
  digitalWrite(3, HIGH);
  delay(10);

  Serial.begin(38400);
  Serial.println("boot");
  delay(10);
  
  wdt_enable(WDTO_8S);
  delay(10);
  config_init();
  
  delay(100);
  
  if(RedFly.init(38400, HIGH_POWER)) {
    debugout("INIT ERR\n"); //there are problems with the communication between the Arduino and the RedFly
    mode = 'S';
  }
  delay(200);
}

void loop() {
  if(mode == 0 || mode == 'C'){
    RedFly.disable();
    startS = millis();
    wdt_reset();

    while(millis()-startS < 5000) {
      int8_t rc = readSerial();
      if(rc == 0) {
        mode = 'S';
        return;
      }
    }
    Serial.println("NO DATA");
    matrix.bytes[30] ^= 0xF;
    RedFly.enable();
  }
  if(mode == 0) {
    uint8_t rssi = RedFly.getrssi();
    if(rssi == 0) {
      if(connect()) {
        matrix.bytes[49] ^= 0x0F;
      }
    } else {
      mode = 'C';
    }
  }
  if(mode == 'W' || mode == 'C') {
    if(millis() - startW > 10000) {
      ping();
      startW = millis();
    }

    startS = millis();
    while(millis()-startS < 1000) {
      int8_t rc = readUDP();
      if(rc == 0) {
        mode = 'W';
      } else if(rc == 2) {
        mode = 0;
      }
    }
  }

  if(mode == 'S') {
    readSerial();
  }
}