#include "matrix.hpp"

// cie = POTENZ((v/15*100+16)/116;3) * 127
//const uint8_t ciePWM[] = {0, 1, 2, 4, 6, 10, 14, 20, 27, 36, 46, 58, 72, 88, 106, 127}; // 7 bit
const uint16_t cieCycles[] = {310, 557, 878, 1272, 1738, 2278, 2890, 3575, 4333, 5164, 6068, 7045, 8094, 9217, 10412}; //4ms

uint8_t currentRow = 0;
uint8_t currentVal = 0;
matrix_t matrix = {
 15, 0, 0,10,15,15, 0,15,15, 2,
 15, 0, 0,15, 0, 0, 0,15, 0,15,
 15, 0, 0,15,15, 0, 0,15, 0,15,
 15, 0, 0,15, 0, 0, 0,15, 0,15,
 13,15, 1,10,15,15, 0,15,15, 2
};
uint8_t n = 0;

void matrix_init() {

  for(int i=4; i<20; i++) {
    pinMode(i, OUTPUT);
    digitalWrite(i, 0);
  }
  n = 1;

  noInterrupts(); // disable all interrupts
  // initialize Timer1
  // for calculating the pixel PWM
  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1  = 0;

  OCR1A = 800; // compare match register 16MHz/1/10kHz
  TCCR1B |= (1 << WGM12); // CTC mode
  TCCR1B |= (1 << CS10); // 1 prescaler
  TIMSK1 |= (1 << OCIE1A); // enable timer compare interrupt
  
  interrupts(); // enable all interrupts
}

ISR(TIMER1_COMPA_vect) {
  //stop on reboot
  if(n == 0) {
    noInterrupts();
    TIMSK1 &= ~(1 << OCIE1A);
    interrupts();
  }
  if(++currentRow >= N_ROWS) {
    currentRow = 0;
    
    if(++currentVal >= 15) {
      currentVal = 0;
    }

    OCR1A = cieCycles[currentVal];
  }

  const row_t& row = matrix.rows[currentRow];
  const uint16_t pixels = 
    (row.bytes[0] > currentVal ? 1 << (N_COLS - 1) : 0) |
    (row.bytes[1] > currentVal ? 1 << (N_COLS - 2) : 0) |
    (row.bytes[2] > currentVal ? 1 << (N_COLS - 3) : 0) |
    (row.bytes[3] > currentVal ? 1 << (N_COLS - 4) : 0) |
    (row.bytes[4] > currentVal ? 1 << (N_COLS - 5) : 0) |
    (row.bytes[5] > currentVal ? 1 << (N_COLS - 6) : 0) |
    (row.bytes[6] > currentVal ? 1 << (N_COLS - 7) : 0) |
    (row.bytes[7] > currentVal ? 1 << (N_COLS - 8) : 0) |
    (row.bytes[8] > currentVal ? 1 << (N_COLS - 9) : 0) |
    (row.bytes[9] > currentVal ? 1 << (N_COLS - 10) : 0);

  writeRow(currentRow, pixels);
}



void writeRow(const uint8_t& nRow, const uint16_t& values) {
  //disable rows
  PORTC = 0x00;
  //set lower 4 values
  PORTD &= 0x0F;
  PORTD |= values << 4;
  //set upper 6 values
  PORTB = values >> 4;
  //enable row
  PORTC = 0b00100000 >> nRow;
}